==> mothra-tools_2.12-1.5.2-sources.jar: packer/PartitionerPackLogic.scala <==
// Copyright 2015-2022 by Carnegie Mellon University
// See license information in LICENSE.txt

package org.cert.netsa.mothra.packer

import org.cert.netsa.io.ipfix.Record


/**
  * An implementation of the [[PackingLogic]] trait.  The `pack()` method
  * stores Records into files based on a sequence of [[Partitioner]]s that is
  * passed to this class's constructor.
  *
  * @param partitioners How records are to be partitioned.
  */
case class PartitionerPackLogic(partitioners: Seq[Partitioner])
    extends PackingLogic
{
  /** The observationDomain to use for newly created files. */
  val domain = 0

  /** The version of the packing logic; this value becomes part of the
    * path in the directory hierarchy. */
  val version = 2

  // Define the pack() method required by our trait.
  // The `packStream()` method (on [[Packer]]) calls it for each
  // IPFIX record to determine how the record is stored.
  def pack(record: Record): TraversableOnce[PackableRecord] = {
    // extract the fields
    val fields: Seq[Option[String]] =
      for (p <- partitioners) yield (p.pathForRecord(record))
    // create the output path
    val relPath = Seq[String](s"v${version}") ++ (
      for { f <- fields ; f2 <- f } yield f2)

    // write the record, updating it if it is a tombstone record
    Tombstone.isTombstoneTemplate(record.template) match {
      case Some(tombObject) =>
        val (tombRec, startTime) = tombObject.updateAccessList(record)
        Option(PackableRecord(
          tombRec, relPath.mkString("/"), startTime, domain))
      case None =>
        Option(PackableRecord(record, relPath.mkString("/"), domain))
    }
  }

}

// @LICENSE_FOOTER@
//
// Copyright 2015-2022 Carnegie Mellon University. All Rights Reserved.
//
// This material is based upon work funded and supported by the
// Department of Defense and Department of Homeland Security under
// Contract No. FA8702-15-D-0002 with Carnegie Mellon University for the
// operation of the Software Engineering Institute, a federally funded
// research and development center sponsored by the United States
// Department of Defense. The U.S. Government has license rights in this
// software pursuant to DFARS 252.227.7014.
//
// NO WARRANTY. THIS CARNEGIE MELLON UNIVERSITY AND SOFTWARE ENGINEERING
// INSTITUTE MATERIAL IS FURNISHED ON AN "AS-IS" BASIS. CARNEGIE MELLON
// UNIVERSITY MAKES NO WARRANTIES OF ANY KIND, EITHER EXPRESSED OR
// IMPLIED, AS TO ANY MATTER INCLUDING, BUT NOT LIMITED TO, WARRANTY OF
// FITNESS FOR PURPOSE OR MERCHANTABILITY, EXCLUSIVITY, OR RESULTS
// OBTAINED FROM USE OF THE MATERIAL. CARNEGIE MELLON UNIVERSITY DOES NOT
// MAKE ANY WARRANTY OF ANY KIND WITH RESPECT TO FREEDOM FROM PATENT,
// TRADEMARK, OR COPYRIGHT INFRINGEMENT.
//
// Released under a GNU GPL 2.0-style license, please see LICENSE.txt or
// contact permission@sei.cmu.edu for full terms.
//
// [DISTRIBUTION STATEMENT A] This material has been approved for public
// release and unlimited distribution. Please see Copyright notice for
// non-US Government use and distribution.
//
// Carnegie Mellon(R) and CERT(R) are registered in the U.S. Patent and
// Trademark Office by Carnegie Mellon University.
//
// This software includes and/or makes use of third party software each
// subject to its own license as detailed in LICENSE-thirdparty.tx
//
// DM20-1143

==> mothra-tools_2.12-1.6.0-sources.jar: packer/PartitionerPackLogic.scala <==
// Copyright 2015-2022 by Carnegie Mellon University
// See license information in LICENSE.txt

package org.cert.netsa.mothra.packer

import org.cert.netsa.io.ipfix.Record


/**
  * An implementation of the [[PackingLogic]] trait.  The `pack()` method
  * stores Records into files based on a sequence of [[Partitioner]]s that is
  * passed to this class's constructor.
  *
  * @param partitioners How records are to be partitioned.
  */
case class PartitionerPackLogic(partitioners: Seq[Partitioner])
    extends PackingLogic
{
  /** The observationDomain to use for newly created files. */
  val domain = 0

  /** The version of the packing logic; this value becomes part of the
    * path in the directory hierarchy. */
  val version = 2

  // Define the pack() method required by our trait.
  // The `packStream()` method (on [[Packer]]) calls it for each
  // IPFIX record to determine how the record is stored.
  def pack(record: Record): Iterator[PackableRecord] = {
    // extract the fields
    val fields: Seq[Option[String]] =
      for (p <- partitioners) yield (p.pathForRecord(record))
    // create the output path
    val relPath = Seq[String](s"v${version}") ++ (
      for { f <- fields ; f2 <- f } yield f2)

    // write the record, updating it if it is a tombstone record
    Tombstone.isTombstoneTemplate(record.template) match {
      case Some(tombObject) =>
        val (tombRec, startTime) = tombObject.updateAccessList(record)
        Iterator(PackableRecord(
          tombRec, relPath.mkString("/"), startTime, domain))
      case None =>
        Iterator(PackableRecord(record, relPath.mkString("/"), domain))
    }
  }

}

// @LICENSE_FOOTER@
//
// Copyright 2015-2022 Carnegie Mellon University. All Rights Reserved.
//
// This material is based upon work funded and supported by the
// Department of Defense and Department of Homeland Security under
// Contract No. FA8702-15-D-0002 with Carnegie Mellon University for the
// operation of the Software Engineering Institute, a federally funded
// research and development center sponsored by the United States
// Department of Defense. The U.S. Government has license rights in this
// software pursuant to DFARS 252.227.7014.
//
// NO WARRANTY. THIS CARNEGIE MELLON UNIVERSITY AND SOFTWARE ENGINEERING
// INSTITUTE MATERIAL IS FURNISHED ON AN "AS-IS" BASIS. CARNEGIE MELLON
// UNIVERSITY MAKES NO WARRANTIES OF ANY KIND, EITHER EXPRESSED OR
// IMPLIED, AS TO ANY MATTER INCLUDING, BUT NOT LIMITED TO, WARRANTY OF
// FITNESS FOR PURPOSE OR MERCHANTABILITY, EXCLUSIVITY, OR RESULTS
// OBTAINED FROM USE OF THE MATERIAL. CARNEGIE MELLON UNIVERSITY DOES NOT
// MAKE ANY WARRANTY OF ANY KIND WITH RESPECT TO FREEDOM FROM PATENT,
// TRADEMARK, OR COPYRIGHT INFRINGEMENT.
//
// Released under a GNU GPL 2.0-style license, please see LICENSE.txt or
// contact permission@sei.cmu.edu for full terms.
//
// [DISTRIBUTION STATEMENT A] This material has been approved for public
// release and unlimited distribution. Please see Copyright notice for
// non-US Government use and distribution.
//
// Carnegie Mellon(R) and CERT(R) are registered in the U.S. Patent and
// Trademark Office by Carnegie Mellon University.
//
// This software includes and/or makes use of third party software each
// subject to its own license as detailed in LICENSE-thirdparty.tx
//
// DM20-1143
